import { Extension } from '@codemirror/state';
import { Dependencies } from './LanguageClient';
import { InfoRenderer, AutocompleteOptions, LinterOptions, DiagnosticRenderer } from './extensions';
export { Dependencies };
export interface FeatureFlags {
    shouldComplete: boolean;
    shouldLint: boolean;
}
export type ClientDependencies = Partial<Dependencies>;
export interface CodeMirrorDependencies {
    /**
     * The infoRenderer is a function that returns a DOM node that contains the documentation
     * for a completion item. Presumably does markdown conversions to DOM nodes.
     *
     * A function that takes a completion object and returns a DOM node.
     */
    infoRenderer?: InfoRenderer;
    /**
     * The diagnosticRenderer is a function that returns a DOM node that
     * contains the content of a diagnostic. It overrides the default
     * rendering logic for diagnostics.
     */
    diagnosticRenderer?: DiagnosticRenderer;
    /**
     * Say you wanted to change the settings of the `autocomplete` extension,
     * you'd do it with that.
     */
    autocompleteOptions?: AutocompleteOptions;
    /**
     * Say you wanted to change the settings of the `linter` extension,
     * you'd do it with that.
     */
    linterOptions?: LinterOptions;
}
export declare class CodeMirrorLanguageClient {
    private readonly worker;
    private readonly client;
    private readonly infoRenderer;
    private readonly diagnosticRenderer;
    private readonly autocompleteExtension;
    private readonly linterExtension;
    constructor(worker: Worker, { log }?: ClientDependencies, { infoRenderer, diagnosticRenderer, autocompleteOptions, linterOptions, }?: CodeMirrorDependencies);
    start(): Promise<void>;
    stop(): Promise<void>;
    extension(fileUri: string, { shouldLint, shouldComplete }?: FeatureFlags): Extension[];
}
