import { MarkupKind } from 'vscode-languageserver-protocol';
import { LanguageClient } from './LanguageClient';
import { clientFacet, fileUriFacet, textDocumentSync, lspLinter, liquidHTMLCompletionExtension, infoRendererFacet, diagnosticRendererFacet, } from './extensions';
/**
 * The client capabilities are how we tell the language server what
 * features we support so that they have the opportunity to skip doing
 * things that the client does not support. Everything is false by default,
 * but we migth need to change that and this is where we'll do it.
 */
const clientCapabilities = {
    textDocument: {
        completion: {
            completionItem: {
                insertReplaceSupport: true,
                documentationFormat: [MarkupKind.PlainText, MarkupKind.Markdown],
                commitCharactersSupport: false,
            },
        },
    },
};
const defaultLogger = console.log.bind(console);
// There is one LanguageClient
// There is one LanguageServer
// There are many CodeMirror instances
export class CodeMirrorLanguageClient {
    constructor(worker, { log = defaultLogger } = {}, { infoRenderer, diagnosticRenderer, autocompleteOptions, linterOptions, } = {}) {
        this.worker = worker;
        this.client = new LanguageClient(worker, {
            clientCapabilities,
            log,
        });
        this.worker = worker;
        this.infoRenderer = infoRenderer;
        this.diagnosticRenderer = diagnosticRenderer;
        this.autocompleteExtension = liquidHTMLCompletionExtension(autocompleteOptions);
        this.linterExtension = lspLinter(linterOptions);
    }
    async start() {
        await this.client.start();
    }
    async stop() {
        try {
            await this.client.stop();
        }
        finally {
            this.worker.terminate();
        }
    }
    extension(fileUri, { shouldLint, shouldComplete } = { shouldLint: true, shouldComplete: true }) {
        return [
            clientFacet.of(this.client),
            fileUriFacet.of(fileUri),
            textDocumentSync,
            infoRendererFacet.of(this.infoRenderer),
            diagnosticRendererFacet.of(this.diagnosticRenderer),
        ]
            .concat(shouldLint ? this.linterExtension : [])
            .concat(shouldComplete ? this.autocompleteExtension : []);
    }
}
//# sourceMappingURL=CodeMirrorLanguageClient.js.map